/**
 *   ijimu/commons/util
 *   Requirements: Prototype
 */
if (!window.ijimu) 
    window.ijimu = {};

if (window.jQuery) 
    jQuery.noConflict();

//debug
(function(){
	function log(value){
		var console = window.console;
		if (console && console.log) {
			try{
				//console.log(value+' ['+window.location.toString()+']');
				if(Prototype.Browser.IE) console.log(value);
				else console.log.apply(console, arguments);
			}catch(e){
				
			}
		}
	}
	
	function error(error, showAlert){
		log(error);
		if(showAlert) ijimu.alert(error.message||error.title||error.details||error);
	}
	
	ijimu.log = log;
	ijimu.error = error;	
})();

//Util
ijimu.Util = {
	/*uuid: (function(){
		/*var counter = 0;
		return function(){
			return '_'+(counter++)+'_'+new Date().getTime();
		};
	})(),*/
	
	isIE: function(version){
		if(!Prototype.Browser.IE) return false;
		if(Object.isUndefined(version)) return true;
		var ua = navigator.userAgent;
		return version == parseInt(ua.substring(ua.indexOf("MSIE")+5));
	},
	
	uuid: function(){
		return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
		    var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
		    return v.toString(16);
		});
	},
	
	getHost: function(url){
		var url = url||window.location.toString();
		var host = url.substring(url.indexOf('://')+3);
		if(host.indexOf('/')>0) host = host.substring(0, host.indexOf('/'));
		return host;
	},
	
	getRoot: function(url){
		url = url||window.location.toString();
		var i = url.indexOf('://');
		if (i > 0) {
			i = url.indexOf('/', i + 3);
			if(i>0) return url.substring(0, i+1);
			return url+'/';
		}
		else if (url.startsWith('/')) return '/';
		else return '';
	},

	getDir: function(url){
		url = url||window.location.toString();
		var i = url.indexOf('?');
		if(i>0) url = url.substring(0, i);
		i = url.lastIndexOf('/');
		if(i>0) return url.substring(0, i+1);
		return '';
	},
	
	fullURL: function(url, base){
		if(!url||url.length<1) return url;
		if(url.indexOf('://')>0) return url;
		if(url.startsWith('/')) base = this.getRoot(base);
		else base = this.getDir(base)
		if(base.endsWith('/')&&url.startsWith('/')) url = url.substring(1);
		if(!base.endsWith('/')&&!url.startsWith('/')) url = '/'+url;
		return base+url;
	},
	
	query: function(search){
		var query = {};
		var search = search||window.location.search;
		if(!search||search.length==0) return query;
		var ss = search.substring(1).split("&");
		for (i=0;i<ss.length;i++) {
			var p = ss[i].split("=");
			query[p[0]] = p.length==2?p[1]:null;
		}
		return query;
	}
};

ijimu.LocalStorage = 
{
	put: function(name, value){
		if (!window.localStorage) return;
		localStorage[name] = Object.toJSON(value);
	},
	
	get: function(name){
		if (!window.localStorage) return null;
		var value = localStorage[name];
		return value?value.evalJSON():'';
	}
};

//TimeFormater
ijimu.TimeFormater = Class.create({
	initialize: function(date){
		if(!date) this.date = new Date();
		else if(date instanceof Date) this.date = date;
		else this.date = new Date(date);
		if(!this.date.getTime()) this.date = new Date();
	},
	
	getTime: function(includeSeconds){
		var str = this.fill(this.date.getHours())+':'+this.fill(this.date.getMinutes());
		if(includeSeconds) str += ':'+this.fill(this.date.getSeconds());
		return str;
	},
	
	getDay: function(separator){
		separator = separator||'/';
		var y = this.date.getFullYear();
		var m = this.fill(this.date.getMonth()+1);
		var d = this.fill(this.date.getDate());
		return y+separator+m+separator+d;
	},
	
	getDayCN: function(){
		var y = this.date.getFullYear();
		var m = this.date.getMonth()+1;
		var d = this.date.getDate();
		return y+'年'+m+'月'+d+'日';
	},
	
	getWeek: function(){
		var w = ijimu.TimeFormater.Week[this.date.getDay()];
		return w;
	},
	
	fill: function(value, length){
		if(!length) length = 2;
		var result = value = value + '';
		for(var i=0; i<length-value.length; i++){
			result = '0'+result;
		}
		return result;
	}
});

ijimu.TimeFormater.Week = ['周日', '周一', '周二', '周三', '周四', '周五', '周六'];

//IntSequence
ijimu.IntSequence = Class.create(
{
	initialize: function(start){
		this.current = start;
    },
	
	next: function(){
		return this.current++;
	}
});

//EventDispatcher
ijimu.EventDispatcher = 
{			
	addEventListener: function(eventName, listener, priority){
		var listeners = this.getEventListeners(eventName);
		if (listeners.indexOf(listener) < 0) {
			listener._event_priority = priority||0;
			listeners.push(listener);
		}
		return this;
	},
	
	onEventOnce: function(name, handler){
		var caller = function(event){
			handler.call(null, event);
			this.removeEventListener(name, caller);
		}.bind(this);
		return this.addEventListener(name, caller);
	},

	removeEventListener: function(eventName, listener){
		if(arguments.length==0){
			this.removeAllEventListeners();
			return this;
		}
		var listeners = this.getEventListeners(eventName);
		if(listener){
			var i = listeners.indexOf(listener);
			if(i>=0) listeners.splice(i, 1);
		}
		else{
			listeners.clear();
		}
		return this;
	},
	
	removeAllEventListeners: function(){
		this._listenerMap = {};
		return this;
	},
	
	dispatchEvent: function(event){
		if(Object.isString(event)) event = {name: event};
		event.target = this;
		event.stop = function(){
			event._stop = true;
		};
		var listeners = this.getEventListeners(event.name);
		listeners = listeners.concat(this.getEventListeners('*'));
		listeners = listeners.sortBy(function(listener){
			return -listener._event_priority;
		});
		for(var i=0; i<listeners.length; i++){
			listeners[i].call(null, event);
			if(event._stop) break;
		}
		return this;
	},
	
	getEventListeners: function(eventName){
		if(!this._listenerMap){
			this._listenerMap = {};
		}
		var listeners = this._listenerMap[eventName];
		if(!listeners){
			listeners = [];
			this._listenerMap[eventName] = listeners;
		}
		return listeners;
	}
};

//LoopArray
ijimu.LoopArray = Class.create(Enumerable, 
{
	initialize: function(array, start){
		this.array = $A(array);
		this.start = start||0;
    },
	
	setStart: function(value){
		this.start = value;
	},
	
	getStart: function(){
		return this.start;
	},
	
	startNext: function(){
		this.startMove(1);
	},
	
	startPrev: function(){
		this.startMove(-1);
	},
	
	startMove: function(value){
		this.start += value
	},
	
	_each: function(iterator) {
		for (var i=0; i<this.size(); i++)
  			iterator(this.get(i));
	},

	get: function(index){
		index = Math.mod(this.start+index, this.array.length);
		return this.array[index];
	},
	
	size: function(){
		return this.array.length;
	}
});


//base class for async request
ijimu.AsyncRequest = 
{
	invokeCallback: function(){
		var params = $A(arguments);
		var name = params.shift();
		var callbacks = this.getCallbacks(name).concat();
		if(this.options){
			params.push(this.options);
			var name = 'on'+name.capitalize();
			if (this.options[name]) callbacks.push(this.options[name]); //make it last!!
		}
		callbacks.each(function(callback){
			try{
				callback.apply(null, params);
			}catch(e){
				ijimu.log(e);
			}
		});
	},
	
	addSuccessHandler: function(handler){
		return this.addCallback('success', handler);
	},
	
	addFailureHandler: function(handler){
		return this.addCallback('failure', handler);
	},
	
	addCompleteHandler: function(handler){
		return this.addCallback('complete', handler);
	},
	
	addCallback: function(name, handler){
		var callbacks = this.getCallbacks(name);
		callbacks.push(handler);
		return this;
	},
	
	getCallbacks: function(name){
		var key = '_callbacks_'+name;
		var callbacks = this[key];
		if(!callbacks) this[key] = callbacks = [];
		return callbacks;
	}
}

/**
 * cookies
 * @param {Object} path
 * @param {Object} domain
 */
ijimu.Cookies = Class.create(
{
    initialize: function(path, domain){
        this.path = path || '/';
        this.domain = domain || null;
    },
	
    // Sets a cookie
    set: function(key, value, days){
        if (typeof key != 'string') {
            throw "Invalid key";
        }
        if (typeof value != 'string' && typeof value != 'number') {
            throw "Invalid value";
        }
        if (days && typeof days != 'number') {
            throw "Invalid expiration time";
        }
        var setValue = key + '=' + escape(new String(value));
        if (days) {
            var date = new Date();
            date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
            var setExpiration = "; expires=" + date.toGMTString();
        }
        else 
            var setExpiration = "";
        var setPath = '; path=' + escape(this.path);
        var setDomain = (this.domain) ? '; domain=' + escape(this.domain) : '';
        var cookieString = setValue + setExpiration + setPath + setDomain;
        document.cookie = cookieString;
    },
    // Returns a cookie value or false
    get: function(key){
        var keyEquals = key + "=";
        var value = false;
        document.cookie.split(';').invoke('strip').each(function(s){
            if (s.startsWith(keyEquals)) {
                value = unescape(s.substring(keyEquals.length, s.length));
                throw $break;
            }
        });
        return value;
    },
    // Clears a cookie
    clear: function(key){
        this.set(key, '', 1);
    },
    // Clears all cookies
    clearAll: function(){
        document.cookie.split(';').collect(function(s){
            return s.split('=').first().strip();
        }).each(function(key){
            this.clear(key);
        }.bind(this));
    }
});


ijimu.Preloader = Class.create(ijimu.EventDispatcher, {
	initialize: function(url, options){
		this.url = url;
		this.options = options||{};
		this.request = new Ajax.Request(url, {
			method: 'get',
			onLoaded: this.loadedHandler.bind(this),
			onInteractive: this.interactiveHandler.bind(this),
			onComplete: this.completeHandler.bind(this)
		});
	},
	
	onComplete: function(listener){
		this.addEventListener('complete', listener);
	},
	
	onProgress: function(listener){
		this.addEventListener('progress', listener);
	},
	
	loadedHandler: function(response){
		this.totals = response.getHeader('Content-Length');
		this.result = '';
		this.dispatchEvent({
			name: 'progress',
			loaded: 0,
			totals: this.totals,
			percent: 0
		});
	},
	
	interactiveHandler: function(response){
		var loaded = response.responseText.length;
		this.dispatchEvent({
			name: 'progress',
			loaded: loaded,
			totals: this.totals,
			percent: Math.round(loaded*100/this.totals),
			content: response.responseText
		});
	},
	
	completeHandler: function(response){
		this.result = response.responseText;
		this.dispatchEvent({
			name: 'progress',
			loaded: this.totals,
			totals: this.totals,
			percent: 100,
			content: response.responseText
		});
		this.dispatchEvent({
			name: 'complete',
			content: response.responseText,
			status: response.status
		});
	}
});
	
//extend Element
(function(){	
	Element.addMethods(
	{
	    //style
		setStyle: (function(){
			var _super = Element.setStyle;
			return function(element, styles){
				if (arguments.length==3&&Object.isString(arguments[1])) {
					var name = arguments[1];
					styles = {};
					styles[name] = arguments[2];
				}
				return _super(element, styles);
			}
		})(),
		
		removeStyle: function(element, name){
			return Element.setStyle(element, name, null);
		},
		
	    setStylePx: function(element, name, value){
			if(Object.isNumber(value)) value = value+'px';
	        return Element.setStyle(element, name, value);
	    },
	    
	    getStyleNum: function(element, name){
	        var value = Element.getStyle(element, name);
			if(Object.isString(value)&&value.endsWith('%')) value = '';
	        value = parseFloat(value);
			if(Object.isNumber(value)&&!isNaN(value)) return value;
			return 0;
	    },

		//position
	    getX: function(element){
	        return Element.getStyleNum(element, 'left');
	    },
	    
		setX: function(element, value){
	        return Element.setStylePx(element, 'left', value);
	    },
		
	    getY: function(element){
	        return Element.getStyleNum(element, 'top');
	    },
	    
		setY: function(element, value){
	        return Element.setStylePx(element, 'top', value);
	    },
		
		setPosition: function(element, x, y){
			return $(element).setX(x).setY(y);
		},
		
		getPosition: function(element){
			element = $(element);
			return {x: element.getX(), y: element.getY()};
		},
		
		getZ: function(element){
	        return Element.getStyleNum(element, 'zIndex');
	    },
	    
		setZ: function(element, value){
	        return Element.setStyle(element, 'zIndex', value);
	    },
	    
	    getBottom: function(element){
	        return Element.getStyleNum(element, 'bottom');
	    },
	    
		setBottom: function(element, value){
	        return Element.setStylePx(element, 'bottom', value);
	    },
	    
	    getRight: function(element){
	        return Element.getStyleNum(element, 'right');
	    },
	    
		setRight: function(element, value){
	        return Element.setStylePx(element, 'right', value);
	    },
		
		anchorTo: function(element, target){
			element = $(element);
			target = $(target);
			var container = $(element).getOffsetParent();
			var containerOffset = container.cumulativeOffset();
			var targetOffset = target.cumulativeOffset();
			var offset = {
				left: targetOffset.left-containerOffset.left,
				top: targetOffset.top-containerOffset.top
			};
			//x
			if(element.getWidth()+offset.left<container.getWidth()){
				element.removeStyle('right');
				element.setX(offset.left);
			}else{
				element.removeStyle('left');
				element.setStylePx('right', container.getWidth()-offset.left);
			}
			//y
			if(element.getHeight()+target.getHeight()+offset.top<container.getHeight()){
				element.removeStyle('bottom');
				element.setY(offset.top+target.getHeight());
			}else{
				element.removeStyle('top');
				element.setStylePx('bottom', container.getHeight()-offset.top);
			}
			return element;
		},
		
		//size
		setWidth: function(element, value){
	        element = $(element);
			var boundary = element.getStyleNum('paddingLeft')
				+ element.getStyleNum('paddingRight')
				+ element.getStyleNum('borderLeftWidth')
				+ element.getStyleNum('borderRightWidth');
			element.setStylePx('width', Math.max(0, value)-boundary);
	        return element;
	    },
		
	    setHeight: function(element, value){
	        element = $(element);
			var boundary = element.getStyleNum('paddingTop')
				+ element.getStyleNum('paddingBottom')
				+ element.getStyleNum('borderTopWidth')
				+ element.getStyleNum('borderBottomWidth');
	        element.setStylePx('height', Math.max(0, value-boundary));
	        return element;
	    },
	    
	    setSize: function(element, width, height){
	        element = $(element);
	        element.setWidth(width).setHeight(height);
	        return element;
	    },
	    
		getSize: function(element){
			element = $(element);
	        return {width: element.getWidth(), height: element.getHeight()};
	    },
		
		//insert
		appendTo: function(element, toElement){
			element = $(element);
			$(toElement).insert({
				bottom: element
			});
			return element;
		},

		prependTo: function(element, toElement){
			element = $(element);
			$(toElement).insert({
				top: element
			});
			return element;
		},
		
		//content
		setText: function(element, text){
			element = $(element);
			if(!Object.isValued(text)) return element;
			text = (text+'').escapeHTML();
			return element.update(text);
		},
		
		getText: function(element){
			return $(element).innerHTML.escapeHTML();
		},
		
		setHTML: function(element, html){
			return Element.update(element, html);
		},
		
		getHTML: function(element){
			return $(element).innerHTML;
		},
		
		deepClone: function(element) {
			element = $(element);
			var clone = element.clone(true); //has some bug in ie :(
			//remove id
			clone.removeAttr('id');
			clone.select('*').invoke('removeAttr', 'id');
			return clone;
		},
		 
		//others
	    setSelectable: function(element, enabled){
	        element = $(element);
	        if (enabled) {
				element.setAttr('unselectable', 'off')
					.setStyle('WebkitUserSelect', 'text')
					.setStyle('MozUserSelect', 'text');
			}
			else {
				element.setAttr('unselectable', 'on')
					.setStyle('WebkitUserSelect', 'none')
					.setStyle('MozUserSelect', 'none');
			}
	        return element;
	    },
		
		removeAllClass: function(element){
			return Element.setAttr(element, 'class', '');
	    },
		
		removeAttr: function(element, name){
			return Element.setAttr(element, name, null);
		},
		
		setVisible: function(element, value, includeInLayout){
			if(includeInLayout===true){
				if(value) return Element.setStyle(element, 'visibility', 'visible');
				else return Element.setStyle(element, 'visibility', 'hidden');
			}else{
				if(value) return Element.show(element, value);
				else return Element.hide(element);
			}
		},
		
		setId: function(element, value){
			return Element.setAttr(element, 'id', value);
		},
		
		getId: function(element){
			return Element.getAttr(element, 'id');
		},
		
		setSrc: function(element, value){
			if(Object.isUndefined(value)||value==null) return $(element);
			return Element.setAttr(element, 'src', value);
		},
		
		getSrc: function(element){
			return Element.getAttr(element, 'src');
		},
		
		setTitle: function(element, value){
			return Element.setAttr(element, 'title', value);
		},
		
		setAlt: function(element, value){
			return Element.setAttr(element, 'alt', value);
		},
		
		setAltSrc: function(element, value){
			element = $(element);
			var src = element.getAttr('src');
			element.setAttr('src', value);
			new Element('img')
				.setAttr('src', src)
				.onLoad(function(){
					element.setAttr('src', src);
				});
			return element;
		},
		
		setAlpha: function(element, value){
			if(Prototype.Browser.IE){
				//need the child elemnet position relative
				element.setStyle({'filter': 'Alpha(Opacity='+value*100+', Style=0)'});
				//element.setStyle({'filter': 'Gradient(StartColorStr=#55FFFFFF, EndColorStr=#55FFFFFF)'});
			}else{
				element.setStyle('backgroundColor: rgba(255, 255, 255, '+value+')');
			}
			return element;
		},
		
		setEnabled: function(element, value){
			return Element.setAttr(element, 'disabled', value?null:'disabled');
		},
		
		setChecked: function(element, value){
			return Element.setAttr(element, 'checked', value?null:'checked');
		},
		
		//input
		isInput: function(element){
			var tag = $(element).tagName;
			if(tag=='BUTTON') return true;
			if(tag=='INPUT') return true;
			if(tag=='SELECT') return true;
			if(tag=='TEXTAREA') return true;
			return false;
		},
		
		isTextInput: function(element){
			element = $(element);
			if(element.tagName=='INPUT'&&element.getAttr('type')=='text') return true;
			if(element.tagName=='TEXTAREA') return true;
			return false;
		},
		
		/*requestFocus: function(element){
			element = $(element);
			if(window.Android) Android.getWindowManager().requestFocus();
			setTimeout(function(){
				//window.focus();
				element.focus();
				if(element.isTextInput()&&window.Android) Android.getWindowManager().showKeybroad();
			}, 1000);
			return element;
		},*/
		
		//for short
		setAttr: Element.writeAttribute,
		getAttr: Element.readAttribute,
		addClass: Element.addClassName,
		hasClass: Element.hasClassName,
		removeClass: Element.removeClassName,
		toggleClass: Element.toggleClassName
	});
})();

//extend Form
(function(){
	var methods = {
		//fix ie null string
		setValue: (function(){
			var _super = Form.Element.Methods.setValue;
			return function(element, value){
				return _super(element, value||"");
			}
		})()
	};
	["INPUT", "TEXTAREA", "SELECT"].each(function(tag){
		Object.extend(Element.Methods.ByTag[tag], methods);
	});
})();

//extend Event
(function(){
	Object.extend(Event, {
		DOC_RESIZE: 'doc:resize',
		SHORT_CLICK: 'doc:shortClick',
		MOUSE_LONG_PRESS: 'doc:mouseLongPress',
		MOUSE_MOVE_TOLERANCE : 48,
		KEY_CODE: 'keycode:'
	});
	
	var methods = {
		fireNative: function(element, name, data){
			element = $(element);
		    if (element == document && document.createEvent && !element.dispatchEvent)
				element = document.documentElement;
		    var event;
		    if (document.createEvent) {
				event = document.createEvent('HTMLEvents');
				event.initEvent(name, true, true);
		    } else {
				event = document.createEventObject();
				event.eventType = 'on'+name;
		    }
		    Object.extend(event, data||{});
			event._custom = true;
		    if (document.createEvent) {
				element.dispatchEvent(event);
			}
			else {
				element.fireEvent(event.eventType, event);
			}
			return element;
		},
		
	    onEvent: function(element, name, handler){
	        return Element.observe(element, name, handler);
	    },
	
		onEventOnce: function(element, name, handler){
			var caller = function(event){
				handler.call(null, event);
				Element.stopObserving(element, name, caller);
			};
			return Element.observe(element, name, caller);
		},
		
	    onMouseMove: function(element, handler){
	        return Element.onEvent(element, 'mousemove', handler);
	    },
	    
		onMouseDown: function(element, handler){
	        return Element.onEvent(element, 'mousedown', handler);
	    },
		
		onMouseLongPress: function(element, handler){
			return Element.onEvent(element, Event.MOUSE_LONG_PRESS, handler);
		},
		
		onMouseUp: function(element, handler){
	        return Element.onEvent(element, 'mouseup', handler);
	    },
		
	    onMouseEnter: function(element, handler){
	        return Element.onEvent(element, 'mouseenter', handler);
	    },
	    
	    onMouseOver: function(element, handler, append){
	        return Element.onEvent(element, 'mouseover', handler);
	    },
	    
	    onMouseOut: function(element, handler, append){
	        return Element.onEvent(element, 'mouseout', handler);
	    },
	    
		onMouseLeave: function(element, handler){
	        return Element.onEvent(element, 'mouseleave', handler);
	    },
		
	    onClick: function(element, handler){
	        return Element.onEvent(element, 'click', handler);
	    },
		
		onShortClick: function(element, handler){
			return Element.onEvent(element, Event.SHORT_CLICK, handler);
		},
		
		onDoubleClick: function(element, handler){
	        return Element.onEvent(element, 'dblclick', handler);
	    },
		
		onKeyPress: function(element, handler){
	        return Element.onEvent(element, 'keypress', handler);
		},
		
		onKeyDown: function(element, handler){
	        return Element.onEvent(element, 'keydown', handler);
		},
		
		onKeyUp: function(element, handler){
	        return Element.onEvent(element, 'keyup', handler);
		},
		
		/*onResize: function(element, handler){
	        return Element.onEvent(element, 'resize', handler);
		},*/
		
		onEnter: function(element, handler){
			return Element.onEvent(element, Event.KEY_CODE+'13', handler);
		},
		
		onError: function(element, handler){
	        return Element.onEvent(element, 'error', handler);
		},
		
		onLoad: function(element, handler){
			return Element.onEvent(element, 'load', handler);
		},
		
		onChange: function(element, handler){
			return Element.onEvent(element, 'change', handler);
		}
	};
	
	Element.addMethods(methods);
	Object.extend(Event, methods);
	
	Object.extend(Event, {
		isRightClick: function(event){
			if(Prototype.Browser.WebKit) {
				return event.button==2;
			}else{
				return event.isRightClick();
			}
		}
	});
	
	//resize event
	Event.observe(document.onresize ? document : window, "resize", function(){
		Event.fire(document, Event.DOC_RESIZE);
	});
	
	//extend document
	Object.extend(document, {
		onLoad : function(handler){
			if(document.body){ //have loaded
				handler.call(null);
			}else{
				document.observe('dom:loaded', handler);
			}
			return document;
		},
		
		onResize : function(handler){
			Event.observe(document, Event.DOC_RESIZE, handler);
			return document;
		}
	});
	
	//extend window
	Object.extend(window, {
		onMessage : function(handler){
			Event.observe(window, "message", handler);
			return window;
		},
		
		onResize : function(handler){
			Event.observe(document, Event.DOC_RESIZE, handler);
			return window;
		}
	});
})();

//touch
(function(){	
	var DEBUG = false;
	
	var Touch = {};
	
	var TouchUtil = 
	{
		getLocation: function(event){
			//android 2.2, have not pageX and pageY in touchup event, 
			//so use the last touchmove/touchdown event
			var touches = event.touches;
			if(touches&&touches.length>0){
				return this._lastTouch = touches[0];
			}
			return this._lastTouch||event;
		}
	};
	
	var TouchEventTransform = Class.create(
	{	
		initialize: function(){
			$(document).observe('touchstart', this.start.bind(this));
			$(document).observe('touchmove', this.move.bind(this));
			$(document).observe('touchend', this.end.bind(this));
			$(document).observe('touchup', this.up.bind(this));
			$(document).observe('touchcancel', this.cancel.bind(this));
		},

		start: function(event){
			if(DEBUG) ijimu.log('touch start : '+Event.element(event));
			var element = Event.element(event);
			Event.fireNative(element, 'mousedown', this.getMemo(event));
			//event.stop();
		},
		
		move: function(event){
			if(DEBUG) ijimu.log('touch move : '+Event.element(event));
			var element = event.element();
			Event.fireNative(element, 'mousemove', this.getMemo(event));
			if(!Touch.disableDrag) event.stop();
		},
		
		end: function(event){
			if(DEBUG) ijimu.log('touch end : '+event.element());
			var element = event.element();
			var memo = this.getMemo(event);
			Event.fireNative(element, 'mouseup', memo);
			Event.fireNative(element, 'click', memo);
		},
		
		up: function(event){
			if(DEBUG) ijimu.log('touch up : '+event.element());
			this.end(event);
		},
		
		cancel: function(event){
			if(DEBUG) ijimu.log('touch cancel : '+event.element());
			this.end(event);
		},
		
		getMemo: function(event){
			var location = TouchUtil.getLocation(event);
			var memo = {
				_touch: true,
				which: 1,
				metaKey: null,
				pageX: location.pageX,
				pageY: location.pageY,
				clientX: location.clientX,
				clientY: location.clientY,
				screenX: location.screenX,
				screenY: location.screenY,
				touches: event.touches
			};
			return memo;
		}
	});
	
	var FocusHighLight = Class.create({
		initialize: function(){
			this._blur = this.blur.bind(this);
			$(document).observe('mousedown', this.focus.bind(this));
			$(document).observe('mouseup', this._blur);
		},
		
		focus: function(event){
			var element = this.getFocusTarget(event.element());
			if(!element||element==this.element) return;
			if(this.element) this.blur();
			this.element = element;
			this.rawGgColor = element.style['backgroundColor']; //use style property, not getComputedStyle
			this.rawGBImage = element.style['backgroundImage'];
			element.setStyle('backgroundColor', this.getColor());
			element.setStyle('backgroundImage', 'none');
			$(document).observe('mousemove', this._blur);
		},
		
		blur: function(){
			if(!this.element) return;
			$(document).stopObserving('mousemove', this._blur);
			this.element.setStyle('backgroundColor', this.rawGgColor);
			this.element.setStyle('backgroundImage', this.rawGBImage);
			this.element = null;
		},
		
		getFocusTarget: function(element){
			if(element.getAttr('disabled')) return null;
			if(element.isInput()) return null;
			while(element&&element.up){
				var parent = element.up();
				if(this.isPointer(element)){
					//filter inherited style
					if(!parent) return element;
					if(!this.isPointer(parent)) return element;
				}
				element = parent;
			}
		},
		
		isPointer: function(element){
			return element.getStyle('cursor')=='pointer';
		},
		
		getColor: function(){
			return Touch.highLightColor||'#1b9cce';
		}
	});
	
	var RawMouseEventFilter = Class.create({
		initialize: function(){
			document.addEventListener('click', this.stop.bind(this), true);
			document.addEventListener('dblclick', this.stop.bind(this), true);
			document.addEventListener('mousedown', this.stop.bind(this), true);
			document.addEventListener('mouseup', this.stop.bind(this), true);
			document.addEventListener('mousemove', this.stop.bind(this), true);
		},
		
		stop: function(event){
			if(event._touch||event._custom) return;
			if(DEBUG) ijimu.log('stop raw '+event.type+' : '+event.element());
			event.stopPropagation();
		}
	});
	
	var MouseEventTester = Class.create({
		initialize: function(){
			$(document).observe('click', this.log.bind(this));
			$(document).observe('mousedown', this.log.bind(this));
			$(document).observe('mouseup', this.log.bind(this));
			$(document).observe('mousemove', this.log.bind(this));
		},
		
		log: function(event){
			ijimu.log(event.type+' : '+event.element()+'['+event._touch+']');
		}
	});
	
	//init
	ijimu.Touch = Touch;

	if(!Object.isUndefined(window.ontouchstart)){
		new TouchEventTransform();
		new FocusHighLight();
		new RawMouseEventFilter();
		if(DEBUG) new MouseEventTester();
	}
})();


//event trigger
(function(){
	var MosueTrigger = Class.create({
		initialize: function(){
			this._mouseMove = this.mouseMove.bind(this);
			$(document).observe('mousedown', this.mouseDown.bind(this));
			$(document).observe('mouseup', this.mouseUp.bind(this));
			$(document).observe('click', this.click.bind(this));
		},

		mouseDown: function(event){
			if(Event.isRightClick(event)) return;
			this.targetElement = event.element();
			if(!this.targetElement) return;
			this.downTime = new Date().getTime();
			this.downPointerX = event.pointerX();
			this.downPointerY = event.pointerY();
			this.maxMove = 0;
			this.hasMoved = false;
			$(document).observe('mousemove', this._mouseMove);
			if(this.downHandler) this.downHandler(event);
		},

		mouseMove: function(event){
			var dx = event.pointerX()-this.downPointerX;
			var dy = event.pointerY()-this.downPointerY;
			var d = Math.pow(dx*dx+dy*dy, 0.5);
			this.maxMove = Math.max(this.maxMove, d);
			this.hasMoved = this.maxMove>Event.MOUSE_MOVE_TOLERANCE;
			if(this.hasMoved&&this.moveHandler) this.moveHandler(event);
		},

		mouseUp: function(event){
			Event.stopObserving(document, 'mousemove', this._mouseMove);
			if(!this.targetElement) return;
			if(this.upHandler) this.upHandler(event);
		},
		
		click: function(event){
			if(!this.targetElement) return;
			if(this.clickHandler) this.clickHandler(event);
		}
	});
	
	var ShortClickTrigger = Class.create(MosueTrigger, 
	{	
		upHandler: function(){
			if(this.hasMoved) return;
			if(new Date().getTime()-this.downTime>500) return;
			this.targetElement.fire(Event.SHORT_CLICK);
		}
	});
	
	var LongPressTrigger = Class.create(MosueTrigger, 
	{
		mouseDown: function($super, event){
			$super(event);
		},
		
		downHandler: function(){
			clearTimeout(this.timer);
			this.timer = setTimeout(this.fire.bind(this), 600);
			//ijimu.log('long press mouse down : '+this.timer);
		},
		
		upHandler: function(){
			//ijimu.log('long press mouse up : '+this.timer);
			clearTimeout(this.timer);
		},
		
		moveHandler: function(){
			//ijimu.log('long press mouse move : '+this.timer);
			clearTimeout(this.timer);
		},
		
		fire: function(){
			//ijimu.log('long press fire : '+this.timer);
			this.targetElement.fire(Event.MOUSE_LONG_PRESS);
		}
	});
	
	var KeyCodeTrigger = Class.create({
		initialize: function(){
			$(document).observe('keydown', function(event){
				var code = event.keyCode;
				var element = event.element();
				element.fire(Event.KEY_CODE+code);
			});
		}
	});
	
	new ShortClickTrigger();
	new LongPressTrigger();
	new KeyCodeTrigger();
})();

//extend Object
Object.extend(Object, 
{
	isObject: function(obj){
		return typeof(obj)==="object";
	},
	
	isNull: function(obj){
		return obj===null;
	},
	
	isValued: function(obj){
		return !Object.isUndefined(obj)&&!Object.isNull(obj);
	},
	
	extend: function(destination, source){
		if(!destination) destination = {};
		if(!source) return destination;
		for (var property in source) {
			var value = source[property];
			if(Object.isValued(value)) destination[property] = value;
		}
		return destination;
	},
	
	delegate: function(host, target, funOnly){
		for(var name in target){
			if(!Object.isUndefined(host[name])) continue;
			var value = target[name];
			if(!Object.isFunction(value)&&funOnly) continue; 
			if(Object.isFunction(value)) value = value.bind(target);
			host[name] = value;
		}
		return host;
	},
	
	deepClone: function(obj){
		if(!Object.isValued(obj)) return obj;
		if(!Object.isObject(obj)) return obj;
		if(Object.isElement(obj)) return obj;
		var clone = Object.isArray(obj)?[]:{};
		for(prop in obj) clone[prop] = arguments.callee(obj[prop]);
		return clone;
	}
});

//extend Class
(function(){
	var defs = {};
	
	Object.extend(Class, 
	{
		singleton: function(clazz, lazzy, options){
			return (function(){
				var instance = null;
				if(lazzy===false) instance = new clazz(options);
				clazz.getInstance = function(){
					if(instance==null) instance = new clazz(options);
					return instance;
				};
				return clazz.getInstance;
			})();
		},
		
		addDef: function(def){
			def.refs = def.refs||{};
			defs[def.name] = def;
		},
		
		getDef: function(name){
			if(!name) return null;
			return defs[name];
		},
		
		inject: function(clazz, obj){
			if(!obj||!Object.isObject(obj)) return;
			var def = Object.isString(clazz)?Class.getDef(clazz):clazz;
			if(!def) return;
			if (Object.isArray(obj)) return obj.each(function(item){
				Class.inject(clazz, item);
			});
			for(var p in obj){
				var refClass = def.refs[p];
				if(refClass) Class.inject(refClass, obj[p]);
			}
			if(!def.clazz) return obj;
			return Object.extend(obj, def.clazz.prototype||def.clazz);
		}
	});
})();

//extend Function
Object.extend(Function.prototype,
{
	//export a function to top window, and return the name
	top: (function(){
		var _counter = new Date().getTime();
		return function(autoDestroy){
			var name = '__top_fun_'+(_counter++);
			var thiz = this;
			top[name] = function(){
				thiz.apply(null, arguments);
				if(autoDestroy!==false) delete top[name];
			};
			return name;
		}
	})()
});

//extend String
Object.extend(String.prototype,
{
	evalJSON: (function(){
		var _evalJSON = String.prototype.evalJSON;
		return function(){
			var str = this;
			try{
				return _evalJSON.apply(str, $A(arguments));
			}catch(e){
				str = str.replace(/\\\w/g, ' ');
				str = str.replace(/\s/g, ' ');
				return _evalJSON.apply(str, $A(arguments));
			}
		}
	})()
});

				
//extend Array
Object.extend(Array.prototype,
{
	remove: function(item){
		var i = this.indexOf(item);
		if(i>=0) this.splice(i, 1);
		return i;
	},
	
	insert: function(){
		var args = $A(arguments);
		args.splice(1, 0, 0);
		this.splice.apply(this, args);
	}
});

//extend Math
Object.extend(Math,
{
	mod: function(value, m){
		return (value%m+m)%m;
	}
});

//dialog
(function(){
	function progress(message, container){
		if(!document.body) return;
		if(container&&$(container)){
			container = $(container);
		}else{
			container = document.body;
		}
		var element = new Element('span')
			.setStyle({
				'position': 'absolute',
				'left': '0px',
				'bottom': '0px',
				'display': 'block',
				'width': '100%',
				'height': '20px',
				'lineHeight': '20px',
				'zIndex': '99999999999',
				'color': '#608BB6',
				'backgroundColor': '#B6CAE4',
				'textAlign': 'center'
			})
			.setText(message)
			.appendTo(container);
		var dialog = {
			dismiss: function(){
				element.remove();
			},
			
			setMessage: function(message){
				element.setText(message);
			}
		};
		return dialog;
	}
	
	function notice(options){
		if(!document.body) return;
		if(Object.isString(options)) options = {message: options};
		var element = new Element('span')
			.setStyle({
				'position': 'absolute',
				'left': '0px',
				'bottom': '0px',
				'display': 'block',
				'width': '100%',
				'height': '30px',
				'lineHeight': '30px',
				'zIndex': '99999999999',
				'color': '#6F6F6F',
				'backgroundColor': '#DFDFDF',
				'textAlign': 'center'
			})
			.setText(options.message)
			.appendTo(document.body);
		setTimeout(function(){element.remove();}, options.timeout||2000);
	}
	
	function confirm(options){
		var result = window.confirm(options.message);
		if(result&&options.onOk) options.onOk();
		if(!result&&options.onCancel) options.onCancel();
		if(options.onResult) options.onResult(result);
	}
	
	function alert(){
		window.alert.apply(window, arguments);
	}
	
	ijimu.alert = alert;
	ijimu.confirm = confirm;
	ijimu.notice = notice;
	ijimu.progress = progress;
})();


//android
(function(){
	if(!window.Android) return;
	
	function notice(){
		if(!Object.isString(message)) message = message.message;
		Android.getWindowManager().notice(message);
	}
	
	function progress(message, fun){
		if(!message) message = null;
		var dialog = Android.getWindowManager().progress(message);
		if(fun){
			try{
				fun();
			}
			finally{
				dialog.dismiss();
			}
		}
		return dialog;
	}
	
	ijimu.notice = notice;
	ijimu.progress = progress;
})();
